package gov.anl.aps.dm.sync;

import java.io.IOException;
import java.io.InputStream;
import java.io.FileInputStream;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.util.Properties;
import java.util.logging.FileHandler;
import java.util.logging.Level;
import java.util.logging.Logger;

public class Synchronizer {
    private OracleConnection oConnection;
    private PsqlConnection pConnection;
    private final Properties config = new Properties();
    private final Properties tableRows = new Properties();
    private final String oracleTable = "oracleTable.properties";
    private FileHandler fileHandler;
    
    private static final Logger logger = Logger.getLogger("AccountSynchronizerLogger");

    public static void main(String[] argv) {
        Synchronizer sync = new Synchronizer();
        sync.loadProperties(argv[0]);
        sync.initLogger();
        sync.oConnection = new OracleConnection(logger);
        sync.pConnection = new PsqlConnection(logger);
        sync.connectAndInit();
        sync.synchronize();
    }
    
    private void loadProperties(String configFile) {
        InputStream configInputStream = null;
	try {
               configInputStream = new FileInputStream(configFile);
        } catch (Exception e) {
               e.printStackTrace();
        }    

        if (configInputStream != null) {
            try {
                this.config.load(configInputStream);
            } catch (IOException ex) {
            	initDefaultLogger();
                logger.log(Level.SEVERE, "can't load configuration ", ex);
                System.exit(-1);
            }
        } else {
            initDefaultLogger();
            logger.log(Level.SEVERE, "can't find configuration file");
            System.exit(-1);
        }
        
        InputStream tableInputStream = getClass().getClassLoader().getResourceAsStream(oracleTable);
        if (tableInputStream != null) {
            try {
                this.tableRows.load(tableInputStream);
            } catch (IOException ex) {
                logger.log(Level.SEVERE, "can't load Oracle table rows definitions ", ex);
                System.exit(-1);
            }
        }
   }
    
    private void initDefaultLogger() {
        	try {
    			fileHandler = new FileHandler("accountSynchronizer.log");
    			logger.addHandler(fileHandler);
                logger.log(Level.WARNING, "Using default logger ");
    		} catch (SecurityException | IOException ex) {
    			ex.printStackTrace();
    			System.exit(-1);
    		}    		
    }
    
    private void initLogger() {
    	String pattern = config.getProperty("log.file"); 
    	int limit = 0;
    	int count = 0;
    	try {
    		String limitProperty = config.getProperty("log.limit");
    		limit = Integer.decode(limitProperty);
    		count = Integer.decode(config.getProperty("log.count"));
    	} catch (NumberFormatException e) {
        	initDefaultLogger();
            logger.log(Level.WARNING, "The logger limit or count are not configured properly. ", e);
    		System.exit(-1);
    	}
    	try {
			fileHandler = new FileHandler(pattern, limit, count);
			logger.addHandler(fileHandler);
		} catch (SecurityException | IOException e) {
			e.printStackTrace();
			System.exit(-1);
		}
    }
    
    private void connectAndInit() {
        try {
            pConnection.connect(config);
        }
        catch (SQLException e) {
            System.exit(-1);            
        }
        try {
            oConnection.connect(config);
        }
        catch (SQLException e) {
            pConnection.close();
            System.exit(-1);
        }
        try {
            pConnection.init();
        }
        catch (SQLException e) {
            pConnection.close();
            oConnection.close();
            System.exit(-1);
        }      
    }
    
    private void synchronize() {
        java.util.Calendar cal = java.util.Calendar.getInstance();
        Timestamp current = new Timestamp(cal.getTimeInMillis());
        DmUser dmuser = new DmUser();
        try {
            ResultSet results = oConnection.getUsers(config.getProperty("oracle.database.table"));
            while (results.next()) {
                String badge = results.getString("BADGE_NO");
                String username = config.getProperty("user.userid.prefix") + badge;
                if (pConnection.isUser(username)) {
                	Timestamp userLastUpdate = pConnection.getUserLastUpdate(username);
                    if ((results.getTimestamp("LAST_CHANGE_DATE") == null) || (userLastUpdate == null) || (results.getTimestamp("LAST_CHANGE_DATE").compareTo(userLastUpdate) >= 0)) {
                        // update user
                       dmuser.email = results.getString("EMAIL");
                        dmuser.username = username;
                        dmuser.lastName = results.getString("LAST_NAME");
                       pConnection.updateUser(dmuser, current);
                       dmuser.clear();
                    }
                } else {
                    // add user
                    dmuser.badge = badge;
                    dmuser.email = results.getString("EMAIL");
                    dmuser.firstName = results.getString("FIRST_NAME");
                    dmuser.lastName = results.getString("LAST_NAME");
                    dmuser.middleName = results.getString("MIDDLE_NAME");
                    dmuser.username = username;
                    pConnection.addUser(dmuser, current);
                    dmuser.clear();
                }
            }
         } catch (SQLException ex) {
            logger.log(Level.SEVERE, "table processing error ", ex);
        }
        finally {
            pConnection.close();
            oConnection.close();            
        }
    }
    
}
